#!/usr/bin/env python

import os
import shutil
import sys

from optparse import OptionParser

def header():
    """Setup an array of information before starting the program."""

    # Read authentication file
    import ConfigParser
    config = ConfigParser.RawConfigParser()

    # If $CLUSTERLAUNCHER_CONF is set, we will assume success or total failure
    if os.getenv('CLUSTERLAUNCHER_CONF'):
        configfile = os.getenv('CLUSTERLAUNCHER_CONF')
        if not os.path.exists(configfile):
            sys.stderr.write("ERROR: environ['CLUSTERLAUNCHER_CONF'] is set incorrectly to a file that does not exists.\n")
            sys.stderr.write("Please remove the enviornment variable or set it correctly.\n")
            sys.exit(1)
    else:
        # Look for the configuration file in the same directory as the launcher
        configfile = os.path.join(os.path.dirname(__file__), 'clusterlauncher.conf')
        if not os.path.exists(configfile):
            # Look for the configuration file in the user's home directory
            configfile = os.path.join(os.path.expanduser('~'), '.clusterlauncher.conf')
        if not os.path.exists(configfile):
            # Look for the configuration file in /etc/clusterlauncher
            defaultfile = os.path.join('/etc', 'cassandralauncher', 'clusterlauncher.conf')
            configfile = os.path.join(os.path.expanduser('~'), '.clusterlauncher.conf')
            shutil.copyfile(defaultfile, configfile)
        if not os.path.exists(configfile):
            # Exit since we still have not found the configuration file
            sys.stderr.write("Please setup your authentication configurations. Order of importance:\n")
            sys.stderr.write("    The location as set by $CLUSTERLAUNCHER_CONF.\n")
            sys.stderr.write("    {0}\n".format(os.path.join(os.path.dirname(__file__), 'clusterlauncher.conf')))
            sys.stderr.write("    {0}\n".format(os.path.join(os.path.expanduser('~'), '.clusterlauncher.conf')))
            sys.stderr.write("    {0}\n".format(os.path.join('/etc', 'cassandralauncher', 'clusterlauncher.conf')))
            sys.exit(1)
    
    # Read configuration file
    config.read(configfile)
    config.add_section('Internal')
    config.set('Internal', 'last_location', configfile)

    # Constantly referenced filenames
    KEY_PAIR = config.get('EC2', 'key_pair') if config.has_option('EC2', 'key_pair') else 'DataStaxLauncher'
    KEY_PAIR = KEY_PAIR.replace('.pem', '')

    PEM_HOME = os.path.expanduser(config.get('EC2', 'pem_home'))
    HOST_FILE = os.path.join(PEM_HOME, 'ds_known_hosts')
    PEM_FILE = os.path.join(PEM_HOME, KEY_PAIR) + '.pem'

    # Ensure filesystem access
    if not os.path.exists(PEM_HOME):
        try:
            os.makedirs(PEM_HOME)
        except:
            sys.stderr.write("Failed to create %s. Please ensure the parent directory has write permissions.\n" % PEM_HOME)
            sys.exit(1)
    if not os.access(PEM_HOME, os.W_OK):
        sys.stderr.write("Please ensure %s is writeable. Nothing was launched.\n" % PEM_HOME)
        sys.exit(1)

    return [config, KEY_PAIR, PEM_HOME, HOST_FILE, PEM_FILE]

def printConnections(user, private_ips, public_ips):
    """Helper method for printing ssh commands."""

    # Print SSH commands
    for publicIP in public_ips:
        print '{0} {1}@{2}'.format(config.get('System', 'ssh'), user, publicIP)
    print

    # Print IPs (CSV)
    print "Private IPs:"
    print ", ".join(private_ips)
    print "Public IPs:"
    print ", ".join(public_ips)
    print

def typed_input(type_check, question=''):
    """Like raw_input, but reprompts to ensure strict type checking."""

    toReturn = ""

    # Repeat question until the input is an int
    while not isinstance(toReturn, type_check):
        try:
            toReturn = type_check(raw_input(question))
        except ValueError:
            toReturn = ""
        
    # Return the int
    return toReturn

def choose(question, choices, noneOption=False, sort=True):
    """Like raw_input, but prompts options and reprompts until answer is valid."""

    print question

    # Sort dictionary keys
    if sort:
        choices.sort()

    # Add an option for adding None to the list as 0
    if noneOption:
        choices = ['None'] + choices

    if len(choices) == 0:
        print "No choices available."
        print
        sys.exit()

    # List the options
    for i, choice in enumerate(choices):
        print "  [{0}] {1}".format(i, choice)

    if len(choices) == 1:
        index = 0
        print '%s automatically chosen.' % choices[0]
    else:
        # Repeat question until the input is a valid int
        index = ""
        while not isinstance(index, int) or index >= len(choices):
            try:
                index = int(raw_input())
            except ValueError:
                index = ""
        
    # Clear a line and return the choice
    print
    return choices[index]

def parse_cli_options(options_tree):
    """Allows for a simplified way to parse complex argument types.
    In particular, when coming from different sources."""

    parser = OptionParser()

    # Keys in options_tree dictionary are the arguments
    options = options_tree.keys()
    options.sort()

    for option in options:
        # All options are created from said dictionary
        section = options_tree[option]['Section']
        help = options_tree[option]['Help']
        action = options_tree[option]['Action'] if 'Action' in options_tree[option] else 'store'
        parser.add_option('--{0}'.format(option), action=action,
                          dest='{0}_{1}'.format(section, option),
                          help=help)

    (options, args) = parser.parse_args()
    return vars(options)



#!/usr/bin/env python

import os
import time
import sys

import ec2
import rax
import common

config, KEY_PAIR, PEM_HOME, HOST_FILE, PEM_FILE = common.header()

# Cluster choice dataset
clusterChoices = {
    'EC2':{
        'Ubuntu':{
            'RightScale':{
                '10.10':{
                    'User': 'root',
                    'Manifest': 'RightImage_OSS_Ubuntu_Maverick_x86_64_5.5.9.7_RC3.manifest.xml',
                    'AMI': 'ami-46f0072f'  
                },
                '10.04':{
                    'User': 'root',
                    'Manifest': 'RightImage_Ubuntu_10.04_x64_v5.6.8.1.manifest.xml',
                    'AMI': 'ami-70fb0a19'
                }
            },
            'Canonical':{
                '11.10':{
                    'User': 'ubuntu',
                    'Manifest': 'ubuntu-oneiric-11.10-amd64-server-20111205.manifest.xml',
                    'AMI': 'ami-c162a9a8'
                },
                '11.04':{
                    'User': 'ubuntu',
                    'Manifest': 'ubuntu-natty-11.04-amd64-server-20111003.manifest.xml',
                    'AMI': 'ami-71589518'
                },
                '10.10':{
                    'User': 'ubuntu',
                    'Manifest': 'ubuntu-maverick-10.10-amd64-server-20111001.manifest.xml',
                    'AMI': 'ami-1933fe70'
                },
                '10.04':{
                    'User': 'ubuntu',
                    'Manifest': 'ubuntu-lucid-10.04-amd64-server-20110930.manifest.xml',
                    'AMI': 'ami-1136fb78'
                },
            }
        },
        'CentOS':{
            'RightScale':{
                '5.6':{
                    'User': 'root',
                    'Manifest': 'RightImage_CentOS_5.6_x64_v5.7.14.manifest.xml',
                    'AMI': 'ami-49e32320'
                },
                '5.4':{
                    'User': 'root',
                    'Manifest': 'RightImage_CentOS_5.4_x64_v5.6.8.1.manifest.xml',
                    'AMI': 'ami-9ae312f3'
                }
            }
        },
        'Debian':{
            'RightScale':{
                '6.0':{
                    'User': 'root',
                    'Manifest': 'rightimage_debian_6.0.1_amd64_20110406.1.manifest.xml',
                    'AMI': 'ami-c40df0ad'
                }
            }
        }
    },
    'Rackspace': {
        'Ubuntu':{
            '11.10':{
                'Image': 119
            },
            '11.04':{
                'Image': 115
            },
            '10.10 (deprecated)':{
                'Image': 69
            },
            '10.04':{
                'Image': 112
            }
        },
        'CentOS':{
            '6.0':{
                'Image': 118
            },
            '5.6':{
                'Image': 114
            },
            '5.5 (deprecated)':{
                'Image': 51
            },
            '5.4 (deprecated)':{
                'Image': 187811
            }
        },
        'Debian':{
            '6.0':{
                'Image': 104
            },
            '5.0':{
                'Image': 103
            }
        },
        'Fedora':{
            '16':{
                'Image': 120
            },
            '15':{
                'Image': 116
            },
            '14':{
                'Image': 106
            },
            '13 (deprecated)':{
                'Image': 53
            }
        }
    }
}



def printConnections(user, private_ips, public_ips, pem_file=False):
    # Print SSH commands
    for publicIP in public_ips:
        # Allow for a quicker SSH command into the cluster
        low_security_args = ''
        if config.getboolean('Cluster', 'low_security_ssh'):
            low_security_args = ' -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no'

        if pem_file:
            print '{0}{1} -i {2} {3}@{4}'.format(config.get('System', 'ssh'), low_security_args, pem_file, user, publicIP)
        else:
            print '{0}{1} {2}@{3}'.format(config.get('System', 'ssh'), low_security_args, user, publicIP)
    print

    # Print IPs (CSV)
    print ", ".join(public_ips)
    print ", ".join(private_ips)
    print

def main():
    print "Using configuration file: %s" % config.get('Internal', 'last_location')
    print
    print "Welcome to the Cluster Launcher!"
    print "    The easiest way to interface with Amazon's EC2 and Rackspace's CloudServers"
    print "    and produce a plain instance (or cluster) in under 5 minutes!"
    print

    if not config.get('Shared', 'handle'):
        sys.stderr.write("Ensure {0} is appropriately set.\n".format(config.get('Internal', 'last_location')))
        sys.stderr.write("    'Shared:handle' is missing.\n")
        sys.exit(1)

    cloud = common.choose("Choose your Cloud Testing Host: ", clusterChoices.keys())

    # Ensure access keys are setup
    if cloud == 'EC2':
        if not config.get('EC2', 'aws_access_key_id') or not config.get('EC2', 'aws_secret_access_key'):
            sys.stderr.write("Ensure {0} is appropriately set.\n".format(config.get('Internal', 'last_location')))
            sys.stderr.write("    'EC2:aws_access_key_id|aws_secret_access_key' are missing.\n")
            sys.exit(1)
        if (config.get('EC2', 'aws_access_key_id')[0] == '"' or config.get('EC2', 'aws_secret_access_key')[0] == '"' or
            config.get('EC2', 'aws_access_key_id')[0] == "'" or config.get('EC2', 'aws_secret_access_key')[0] == "'"):
            sys.stderr.write("None of the configurations should be wrapped in quotes.\n")
            sys.stderr.write("    EC2:aws_access_key_id or EC2:aws_secret_access_key appears to be.\n")
            sys.exit(1)
    if cloud == 'Rackspace':
        if not config.get('Rax', 'rax_user') or not config.get('Rax', 'rax_api_key'):
            sys.stderr.write("Ensure {0} is appropriately set.\n".format(config.get('Internal', 'last_location')))
            sys.stderr.write("    'Rax:rax_user|rax_api_key' are missing.\n")
            sys.exit(1)
        if (config.get('Rax', 'rax_user')[0] == '"' or config.get('Rax', 'rax_api_key')[0] == '"' or
            config.get('Rax', 'rax_user')[0] == "'" or config.get('Rax', 'rax_api_key')[0] == "'"):
            sys.stderr.write("None of the configurations should be wrapped in quotes.\n")
            sys.stderr.write("    Rax:rax_user or Rax:rax_api_key appears to be.\n")
            sys.exit(1)

    action = common.choose("Choose your Cloud Command: ", ['Create', 'Destroy'])

    if action == 'Destroy':
        if cloud == 'EC2':
            ec2.terminate_cluster(config.get('EC2', 'aws_access_key_id'), config.get('EC2', 'aws_secret_access_key'), config.get('Shared', 'handle'))
        if cloud == 'Rackspace':
            rax.terminate_cluster(config.get('Rax', 'rax_user'), config.get('Rax', 'rax_api_key'), config.get('Shared', 'handle'))
        sys.exit()

    reservation_size = common.typed_input(int, "Choose your Cluster Size: ")
    print

    operating_system = common.choose("Choose your Testing Operating System: " , clusterChoices[cloud].keys())

    if cloud == 'EC2':
        provider = common.choose("Choose your Image Provider: ", clusterChoices[cloud][operating_system].keys())

        version = common.choose("Choose your Operating System Version: ", clusterChoices[cloud][operating_system][provider].keys())

        image = clusterChoices[cloud][operating_system][provider][version]['AMI']
        user = clusterChoices[cloud][operating_system][provider][version]['User']
        tag = "{5} - {0} Time: {4} {1} {2} Size: {3}".format(provider, operating_system, version, reservation_size, time.strftime("%m-%d-%y %H:%M", time.localtime()), config.get('Shared', 'handle'))

        user_data = raw_input("Enter EC2 user data: ")
        print

        instance_type = config.get('EC2', 'instance_type')
        if not instance_type:
            instance_type = common.choose('Choose your Instance Size:', ['m1.large', 'm1.xlarge', 'm2.xlarge', 'm2.2xlarge', 'm2.4xlarge'])

        clusterinfo = ec2.create_cluster(config.get('EC2', 'aws_access_key_id'), config.get('EC2', 'aws_secret_access_key'),
                                        reservation_size, image, tag, KEY_PAIR,
                                        instance_type, config.get('EC2', 'placement'), PEM_HOME, user_data)
        private_ips, public_ips, reservation = clusterinfo

        printConnections(user, private_ips, public_ips, PEM_FILE)
    
    if cloud == 'Rackspace':
        version = common.choose("Choose your Operating System Version: ", clusterChoices[cloud][operating_system].keys())

        image = clusterChoices[cloud][operating_system][version]['Image']
        tag = "{0} Time {4} {1} {2} Size {3}".format(config.get('Shared', 'handle'), operating_system, version, reservation_size, time.strftime("%m-%d-%y %H:%M", time.localtime())).replace(' ', '-').replace(':', '_').replace('.', '_')

        flavor_array = ['256', '512', '1GB', '2GB', '4GB', '8GB', '15.5GB', '30GB']
        flavor_choice = config.get('Rax', 'flavor')
        if not flavor_choice:
            flavor_choice = common.choose("Choose your Instance Size: ", flavor_array, sort=False)

        flavor_dict = {}
        for i, flavor in enumerate(flavor_array):
            flavor_dict[flavor] = i + 1
            flavor_dict[str(i + 1)] = i + 1 # For backward compliance
        flavor = flavor_dict[flavor_choice]

        private_ips, public_ips = rax.create_cluster(config.get('Rax', 'rax_user'), config.get('Rax', 'rax_api_key'),
                                                  reservation_size, image, tag, flavor)

        printConnections(config.get('Rax', 'user'), private_ips, public_ips)



def run():
    try:
        main()
    except KeyboardInterrupt:
        print
        sys.stderr.write("Program Aborted.\n")
        print
        sys.exit(1)



if __name__ == "__main__":
    run()



#!/usr/bin/env python

import getpass
import os
import re
import shlex
import subprocess
import tempfile
import time
import sys

import ec2
import common


# Globals
private_ips = []
public_ips = []

config, KEY_PAIR, PEM_HOME, HOST_FILE, PEM_FILE = common.header()
cli_options = {}


#################################
# Execution and SSH commands

def exe(command, wait=True):
    """Execute a subprocess command"""

    # Open a subprocess to run your command
    process = subprocess.Popen(shlex.split(str(command)), stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    if wait:
        read = process.communicate()
        return read
    else:
        return process

def create_ssh_cmd(sshuser, host):
    """SSH function that returns SSH string"""

    connstring = "%s -i %s -o UserKnownHostsFile=%s %s@%s " % (
                        config.get('System', 'ssh'), 
                        PEM_FILE, 
                        HOST_FILE,
                        sshuser, host
                    )
    return connstring

def exe_ssh_cmd(connstring, command, wait=True):
    """SSH function that executes SSH string + command"""

    return exe(connstring + ' "' + command + '"', wait)

def scp_send(sshuser, host, sendfile, tolocation=''):
    """SCP send a file"""

    return exe("%s -i %s -o UserKnownHostsFile=%s %s %s@%s:%s" % (
                config.get('System', 'scp'),
                PEM_FILE,
                HOST_FILE,
                sendfile, sshuser, host, tolocation
            ))

#################################

#################################
# Keyless SSH Creation

def sshprompt(prompt, choices):
    """Helper for prompting that ensures user answer is included in choices.
    'prompt' text is displayed after initial blank prompt, in the event that the answer is not included in the choices.
    """

    # Repeat question until the input is a valid int
    while True:
        response = raw_input().lower()
        if response in choices:
            break
        print prompt,
    return response

def prompt_rsa_fingerprints(ip, fingerprint):
    """Prompts the user to accept RSA fingerprints."""

    # Generate and prompt
    securityText = [
        "The authenticity of host '{0} ({0})' can't be established.",
        "RSA key fingerprint is {1}.",
        "Are you sure you want to continue connecting (yes/no/all)?"
    ]
    print "\n".join(securityText).format(ip, fingerprint),
    accept_rsa_fingerprints = sshprompt("Please type 'yes' or 'no' or 'all': ", ['yes', 'no', 'all'])

    # Allow one more chance if answered 'no'
    if accept_rsa_fingerprints == 'no':
        while True:
            confirmation = raw_input("Do you really want to cancel OpsCenter Agent Installation (y/n)? ").lower()
            if confirmation in ['y', 'n']:
                break
        if confirmation == 'y':
            sys.exit(1)
        else:
            print "\n".join(securityText),
            accept_rsa_fingerprints = sshprompt("Please type 'yes' or 'no' or 'all': ", ['yes', 'no', 'all'])
    if accept_rsa_fingerprints == 'no':
        sys.exit(1)

    return accept_rsa_fingerprints

def create_keyless_ssh_ring(public_ips, user):
    """Create keyless SSH ring from primed servers"""

    print "Creating a keyless SSH ring..."

    # Send files to EC2 once
    kicker_conn = create_ssh_cmd(user, public_ips[0])
    scp_send(user, public_ips[0], PEM_FILE)
    scp_send(user, public_ips[0], HOST_FILE)
    exe_ssh_cmd(kicker_conn, "mv %s .ssh/id_rsa; chmod 400 .ssh/id_rsa" % (KEY_PAIR + '.pem'))
    exe_ssh_cmd(kicker_conn, "mv %s .ssh/known_hosts; chmod 600 .ssh/known_hosts" % 'ds_known_hosts')

    # Transfer files from kicker node to all other nodes
    for node in public_ips[1:]:
        exe_ssh_cmd(kicker_conn, "scp .ssh/id_rsa %s:.ssh/id_rsa" % node)
        exe_ssh_cmd(kicker_conn, "scp .ssh/known_hosts %s:.ssh/known_hosts" % node)

def prime_connections(public_ips, user):
    """Ask acceptance of RSA keys."""

    print "Priming connections..."

    # Clear previous file
    with open(HOST_FILE, 'w') as f:
        f.write('')

    accept_rsa_fingerprints = config.get('Cassandra', 'accept_rsa_fingerprints')

    # Prompt for RSA fingerprint authentication for each IP
    for ip in public_ips:

        while True:
            # Get public RSA key
            rsa_key = exe('ssh-keyscan -t rsa {0}'.format(ip))[0]
            with tempfile.NamedTemporaryFile() as tmp_file:
                tmp_file.write(rsa_key)
                tmp_file.flush()

                # Generate fingerprint
                fingerprint = exe('ssh-keygen -l -f {0}'.format(tmp_file.name))[0].split()[1]

            # Ensure that stderr didn't return the error message '* is not a public key file.'
            if fingerprint != 'is':
                break

        # If performing individual authentication, prompt
        if accept_rsa_fingerprints != 'all':
            accept_rsa_fingerprints = prompt_rsa_fingerprints(ip, fingerprint)
        
        # Append all public RSA keys into HOST_FILE
        with open(HOST_FILE, 'a') as f:
            f.write(rsa_key)

        # Clear authentication if not marked 'all'
        if accept_rsa_fingerprints != 'all':
            accept_rsa_fingerprints = ''

    create_keyless_ssh_ring(public_ips, user)

#################################

#################################
# Installing OpsCenter Agents

def install_opsc_agents(user):
    """Wait for OpsCenter agent tarball to be created. Then install the agent on all other nodes."""

    # Give AWS some time to warm up
    wait = 10
    print "Waiting %s seconds for EC2 instances to warm up..." % wait
    time.sleep(wait)

    # Authenticate and ring cluster with keyless SSH
    prime_connections(public_ips, user)

    if check_cascading_options('installopscenter', optional=True) != 'False':
        # Connection to the OpsCenter machine to be used later
        opsc_conn = create_ssh_cmd(user, public_ips[0])

        print "Waiting for the agent tarball to be created (This can take up to 4 minutes)..."
        print "    If taking longer, ctrl-C and login to AMI to see error logs."
        while exe_ssh_cmd(opsc_conn, "ls /usr/share/opscenter/agent/opscenter-agent.tar.gz")[1]:
            # The agent tarball has yet to be created
            time.sleep(5)

        # Ensures the tarball is fully written before transferring
        while True:
            old_md5 = exe_ssh_cmd(opsc_conn, "md5sum /usr/share/opscenter/agent/opscenter-agent.tar.gz")[0]
            time.sleep(2)
            new_md5 = exe_ssh_cmd(opsc_conn, "md5sum /usr/share/opscenter/agent/opscenter-agent.tar.gz")[0]

            if old_md5 == new_md5:
                break

        print "Installing OpsCenter Agents..."
        for i, node in enumerate(public_ips):
            node_conn = create_ssh_cmd(user, node)

            # Copying OpsCenter Agent tarball
            exe_ssh_cmd(opsc_conn, "scp /usr/share/opscenter/agent/opscenter-agent.tar.gz %s:opscenter-agent.tar.gz" % node)

            # Untarring tarball
            exe_ssh_cmd(node_conn, "sudo mv -f opscenter-agent.tar.gz /usr/share; cd /usr/share; sudo tar --overwrite -xzf opscenter-agent.tar.gz; sudo rm -f opscenter-agent.tar.gz")

            # Starting installation of OpsCenter Agent
            exe_ssh_cmd(node_conn, "cd /usr/share/opscenter-agent/; sudo nohup bin/install_agent.sh opscenter-agent.deb %s %s" % (private_ips[0], private_ips[i]), wait=False)
    print

#################################

#################################
# Log code for private stats

def running_log(reservation, demotime):
    """Logs usage data for personal stats."""

    loginfo = [
        'Running' if config.get('Cassandra', 'demo') == 'True' else 'Ignore',
        config.get('Shared', 'handle'),
        str(demotime),
        str(time.time()),
        str(reservation.id)
    ]
    logline = ",".join(loginfo) + '\n'

    with open('running.log', 'a') as f:
        f.write(logline)

#################################

#################################
# Argument parsing

# This holds the information for all options
# and their metadata to allow easier command
# line argument, configuration reading, and
# raw_input prompting.

options_tree = {
    'handle': {
        'Section': 'Shared',
        'Prompt': 'ShortName (Handle)',
        'Help': 'Your Personal Shortname'
    },
    'aws_access_key_id': {
        'Section': 'EC2',
        'Prompt': 'AWS Access Key ID',
        'Help': 'AWS Access Key ID'
    },
    'aws_secret_access_key': {
        'Section': 'EC2',
        'Prompt': 'AWS Secret Access Key',
        'Help': 'AWS Secret Access Key'
    },
    'clustername': {
        'Section': 'Cassandra',
        'Prompt': 'Cluster Name',
        'Help': 'Name of the Cluster'
    },
    'totalnodes': {
        'Section': 'Cassandra',
        'Prompt': 'Total Nodes',
        'Help': 'Number of Nodes in the Cluster'
    },
    'version': {
        'Section': 'Cassandra',
        'Prompt': 'Version:',
        'Help': 'Community | Enterprise'
    },
    'installopscenter': {
        'Section': 'Cassandra',
        'Prompt': 'Install Opscenter',
        'Help': 'True | False'
    },
    'username': {
        'Section': 'Cassandra',
        'Prompt': 'DataStax Username',
        'Help': 'DataStax Username'
    },
    'password': {
        'Section': 'Cassandra',
        'Prompt': 'Uses getpass()',
        'Help': 'DataStax Password'
    },
    'realtimenodes': {
        'Section': 'Cassandra',
        'Prompt': 'Realtime (Non-Analytic) Nodes',
        'Help': 'Number of Vanilla Cassandra Nodes'
    },
    'cfsreplicationfactor': {
        'Section': 'Cassandra',
        'Prompt': 'CFS Replication Factor',
        'Help': 'CFS Replication Factor'
    },
    'demotime': {
        'Section': 'Cassandra',
        'Prompt': 'Time (in hours) for the cluster to live',
        'Help': 'For use with DemoService'
    },
    'instance_type': {
        'Section': 'EC2',
        'Prompt': 'EC2 Instance Size:',
        'Help': 'm1.large | m1.xlarge | m2.xlarge | m2.2xlarge | m2.4xlarge'
    },
    'noprompts':{
        'Section': 'CLI',
        'Prompt': 'NoPrompts',
        'Action': 'store_true',
        'Help': 'Ensures that no user prompts will occur.'
    }
}

def type_checker(option, read_option, type_check, passive=False):
    """Ensures that the data read is of expected type."""

    if type_check:
        try:
            read_option = type_check(read_option)
        except:
            if passive:
                return None
            sys.stderr.write('"{0}" was expected to be of {1}\n'.format(option, type_check))
            sys.exit(1)
    return read_option

def basic_option_checker(read_option, option, type_check, choices):
    """Performs basic checks on configuration and command line argument data."""

    if read_option:
        read_option = type_checker(option, read_option, type_check)
        if choices:
            if any(read_option.lower() == choice.lower() for choice in choices):
                return read_option
        else:
            return read_option

def check_cascading_options(option, type_check=False, choices=False, password=False, optional=False):
    """Reads from the command line arguments, then configuration file, then prompts
    the user for program options."""
    section = options_tree[option]['Section']

    # Read from sys.argv
    read_option = cli_options['{0}_{1}'.format(section, option)]
    read_option = basic_option_checker(read_option, option, type_check, choices)
    if read_option != None:
        return read_option

    # Read from configfile
    if config.has_option(section, option):
        read_option = config.get(section, option)
        read_option = basic_option_checker(read_option, option, type_check, choices)
        if read_option != None:
            return read_option

    if optional:
        return False

    # Exit(1) if you asked for --noprompts and didn't fill in all variables
    if cli_options['CLI_noprompts']:
        sys.stderr.write('Prompt occurred after specifying --noprompts.\n')
        sys.stderr.write('Missing configuration for "--{0}".\n'.format(option))
        sys.exit(1)

    # Prompt for password if special case
    if password:
        return getpass.getpass()

    # Prompt the user with raw_input or common.choose
    while True:
        prompt = options_tree[option]['Prompt']
        if choices:
            response = common.choose(prompt, choices)
        else:
            response = raw_input('{0}: '.format(prompt))
        response = type_checker(option, response, type_check, passive=True)
        if response != None:
            break

    # Set config to avoid double prompting later (doesn't actually write to disk)
    config.set(section, option, response)
    return response


#################################


def main():
    print "Using configuration file: %s" % config.get('Internal', 'last_location')
    print
    print "Welcome to DataStax' Cassandra Cluster Launcher!"
    print "    The easiest way to get Apache Cassandra up and running in Amazon's EC2"
    print "    in under 5 minutes!"
    print

    global cli_options
    cli_options = common.parse_cli_options(options_tree)

    # Required handle for log purposes and future shared EC2 purposes
    check_cascading_options('handle')

    # Prompt the user with any outstanding running clusters
    if (check_cascading_options('aws_access_key_id')[0] == '"' or check_cascading_options('aws_secret_access_key')[0] == '"' or
        check_cascading_options('aws_access_key_id')[0] == "'" or check_cascading_options('aws_secret_access_key')[0] == "'"):
        sys.stderr.write("None of the configurations should be wrapped in quotes.\n")
        sys.stderr.write("    EC2:aws_access_key_id or EC2:aws_secret_access_key appears to be.\n")
        sys.exit(1)
    ec2.terminate_cluster(check_cascading_options('aws_access_key_id'), check_cascading_options('aws_secret_access_key'), check_cascading_options('handle'), prompt_continuation=True)

    # Get basic information for both Community and Enterprise clusters
    clustername = check_cascading_options('clustername')
    clustername = clustername.replace("'", "").replace(' ', '') # AMI 2.2 will allow spaces

    # Ensure totalnodes > 0
    while True:
        totalnodes = check_cascading_options('totalnodes', int)
        if totalnodes > 0:
            break
    
    version = check_cascading_options('version', choices=['Community', 'Enterprise'])
    user_data = '--clustername %s --totalnodes %s --version %s' % (clustername, totalnodes, version)

    if version.lower() == 'Enterprise'.lower():
        # Get additional information for Enterprise clusters
        username = check_cascading_options('username')
        password = check_cascading_options('password', password=True)

        # Check the number of Vanilla Cassandra nodes that will launch
        while True:
            realtimenodes = check_cascading_options('realtimenodes', int)
            if realtimenodes <= totalnodes:
                break
            else:
                # Clear the previous cfsreplicationfactor
                config.set('Cassandra', 'realtimenodes')
        
        user_data += ' --username %s --password %s --realtimenodes %s' % (username, password, realtimenodes)

        # If Hadoop enabled nodes are launching, check the CFS replication factor
        analyticnodes = totalnodes - realtimenodes
        if analyticnodes > 0:
            while True:
                cfsreplicationfactor = check_cascading_options('cfsreplicationfactor', int)
                if cfsreplicationfactor >= 1 and cfsreplicationfactor <= analyticnodes:
                    break
                else:
                    # Clear the previous cfsreplicationfactor
                    config.set('Cassandra', 'cfsreplicationfactor')
        
            user_data += ' --cfsreplicationfactor %s' % (cfsreplicationfactor)
        print

    # Included for the experimental DemoService that requires demoservice.py to always be running
    demotime = -1
    if config.get('Cassandra', 'demo') == 'True':
        print "Your configuration file is set to launch a demo cluster for a specified time."
        demotime = check_cascading_options('demotime', float)
        print "If the demo service is running, this cluster will live for %s hour(s)." % demotime
        print

    if check_cascading_options('installopscenter', optional=True) == 'False':
        user_data += ' --opscenter no'

    # DataStax AMI specific options and formatting
    image = 'ami-fd23ec94'
    tag = '{0} - DataStaxAMI Time: {1} Size: {2}'.format(check_cascading_options('handle'), time.strftime("%m-%d-%y %H:%M", time.localtime()), totalnodes)
    user = 'ubuntu'

    # Launch the cluster
    instance_type = check_cascading_options('instance_type', choices=['m1.large', 'm1.xlarge', 'm2.xlarge', 'm2.2xlarge', 'm2.4xlarge'])
    clusterinfo = ec2.create_cluster(check_cascading_options('aws_access_key_id'), check_cascading_options('aws_secret_access_key'),
                                    totalnodes, image, tag, KEY_PAIR,
                                    instance_type, config.get('EC2', 'placement'), PEM_HOME,
                                    user_data, cli_options['CLI_noprompts'])

    # Save IPs
    global private_ips
    global public_ips
    private_ips, public_ips, reservation = clusterinfo

    # Log clusterinfo
    running_log(reservation, demotime)

    # Print SSH commands
    print 'Unprimed Connection Strings:'
    for publicIP in public_ips:
        print '{0} -i {1} {2}@{3}'.format(config.get('System', 'ssh'), PEM_FILE, user, publicIP)
    print

    if check_cascading_options('installopscenter', optional=True) != 'False':
        # Print OpsCenter url
        print "OpsCenter Address:"
        print "http://%s:8888" % public_ips[0]
        print "Note: You must wait 60 seconds after Cassandra becomes active to access OpsCenter."
        print

    install_opsc_agents(user)

    print 'Primed Connection Strings:'
    for node in public_ips:
        print '{0} -i {1} -o UserKnownHostsFile={2} {3}@{4}'.format(config.get('System', 'ssh'), PEM_FILE, HOST_FILE, user, node)
    print

    ec2.terminate_cluster(check_cascading_options('aws_access_key_id'), check_cascading_options('aws_secret_access_key'), check_cascading_options('handle'))

def run():
    try:
        main()
    except KeyboardInterrupt:
        print
        sys.stderr.write("Program Aborted.\n")
        print
        sys.exit(1)



if __name__ == "__main__":
    run()



#!/usr/bin/env python

import os
import re
import time
import sys

try:
    from cloudservers import CloudServers
    from cloudservers import Image
except:
    sys.stderr.write("'cloudservers' is not installed on your system. Please run:\n")
    sys.stderr.write("    pip install python-cloudservers\n")
    sys.stderr.write("and try again.\n")
    sys.exit(1)

import common

def create_cluster(rax_user, rax_api_key, reservation_size, image, tag, flavor):

    # Create the Rackspace cluster
    print 'Starting a Rackspace cluster of flavor {0} with image {1}...'.format(flavor, image)
    cloudservers = CloudServers(rax_user, rax_api_key)
    servers = []
    print 'Launching cluster...'
    for i in range(reservation_size):
        transfer_files = {
            '/root/.ssh/authorized_keys' : open(os.path.expanduser("~/.ssh/id_rsa.pub"), 'r')
            # '/root/.ssh/id_rsa.pub' : open(os.path.expanduser("~/.ssh/id_rsa.pub"), 'r')
            # '/root/.ssh/id_rsa' : open(os.path.expanduser("~/.ssh/id_rsa"), 'r')
        }

        servers.append(
            cloudservers.servers.create(
                image=image,
                flavor=flavor,
                name=tag,
                files=transfer_files
            )
        )
    
    print 'Waiting for cluster...'
    time.sleep(10)
    for server in servers:
        while cloudservers.servers.get(server.id).status != "ACTIVE":
            time.sleep(3)
    print "Cluster booted successfully!"

    # Print SSH commands
    public_ips = []
    private_ips = []
    for server in servers:
        public_ips.append(server.public_ip)
        private_ips.append(server.private_ip)
    print

    return [private_ips, public_ips]

def terminate_cluster(rax_user, rax_api_key, search_term):

    # Grab all the infomation for clusters spawn by this tool that are still alive
    ds_reservations = {}
    cloudservers = CloudServers(rax_user, rax_api_key)
    serverlist = cloudservers.servers.list()

    p = re.compile("{0}.*".format(search_term))

    for server in serverlist:
        if p.match(server.name):
            try:
                if cloudservers.servers.get(server.id).status == "ACTIVE":
                    if not server.name in ds_reservations:
                        ds_reservations[server.name] = {
                            'Servers': []
                        }
                    ds_reservations[server.name]['Servers'].append(server)
            except:
                # Server was recently shut down
                pass

    # Prompt for cluster to destroy
    selection = common.choose("Choose the cluster to destroy:", ds_reservations.keys())

    # Confirm cluster termination
    print "Confirm you wish to terminate {0} by pressing 'y'.".format(selection)
    if raw_input().lower() != 'y':
        print "Cluster was not terminated."
        print
        sys.exit()
    print

    # The actual termination command
    for server in ds_reservations[selection]['Servers']:
        try:
            server.delete()
        except:
            print "This server appears to have already been shut down. Please give it some time for the API to adjust."

    print "Termination command complete."
    print

# NOTES:
# cloudservers = CloudServers(user, apiKey)
# 
# Find Flavor List
# for i in cloudservers.flavors.list():
#     print "ID: %s = %s" % (i.id, i.name)
# 
# Find Image List
# for i in cloudservers.images.list():
#     print "ID: %s = %s" % (i.id, i.name)



#!/usr/bin/env python

import os
import stat
import sys
import time

try:
    import boto
except:
    sys.stderr.write("'boto' is not installed on your system. Please run:\n")
    sys.stderr.write("    pip install boto\n")
    sys.stderr.write("and try again.\n")
    sys.exit(1)

from xml.dom.minidom import parseString

import common

def authorize(security_group, port, realm, port_end_range=False):

    # Setup single ports, unless noted
    if not port_end_range:
        port_end_range = port
    
    # Catch errors from overpopulating
    try:
        # Open ports publicly
        if realm == 'public':
            security_group.authorize('tcp', port, port_end_range, '0.0.0.0/0')
        # Open ports privately
        elif realm == 'private':
            security_group.authorize('tcp', port, port_end_range, src_group=security_group)
        # Error out if code was changed
        else:
            sys.stderr.write('Unknown realm assigned!\n')
            sys.exit(1)
    except boto.exception.EC2ResponseError:
        # Continue since ports were probably trying to be overwritten
        pass

def print_boto_error():
    """Attempts to extract the XML from boto errors to present plain errors with no stacktraces."""

    try:
        quick_summary, null, xml = str(sys.exc_info()[1]).split('\n')
        error_msg = parseString(xml).getElementsByTagName('Response')[0].getElementsByTagName('Errors')[0].getElementsByTagName('Error')[0]
        print
        sys.stderr.write('AWS Error: {0}\n'.format(quick_summary))
        sys.stderr.write('{0}: {1}\n'.format(error_msg.getElementsByTagName('Code')[0].firstChild.data, error_msg.getElementsByTagName('Message')[0].firstChild.data))
        print
    except:
        # Raise the exception if parsing failed
        raise
    sys.exit(1)



def create_cluster(aws_access_key_id, aws_secret_access_key, reservation_size, image, tag, key_pair, instance_type, placement, pem_home, user_data=None, noprompts=False):

    # Connect to EC2
    print 'Starting an EC2 cluster of type {0} with image {1}...'.format(instance_type, image)
    conn = boto.connect_ec2(aws_access_key_id, aws_secret_access_key)

    # Ensure PEM key is created
    try:
        print "Ensuring DataStax pem key exists on AWS..."
        key = conn.get_all_key_pairs(keynames=[key_pair])[0]

        print "Ensuring DataStax pem key exists on filesystem..."
        # Print a warning message if the pem file can't be found
        pem_file = os.path.join(pem_home, key_pair + '.pem')
        if os.path.isfile(pem_file):
            print "Ensuring DataStax pem key's permissions are acceptable..."
            # Print a warning message is the pem file does not have the correct file permissions
            permissions = os.stat(pem_file).st_mode
            if not (bool(permissions & stat.S_IRUSR) and     # Ensure owner can read
                    not bool(permissions & stat.S_IRWXG) and # Ensure that group has no permissions
                    not bool(permissions & stat.S_IRWXO)):   # Ensure that others have no permissions

                sys.stderr.write("WARNING: The pem file has permissions that are too open!\n\n")

                if noprompts:
                    sys.exit(1)
                if raw_input("Do you wish to reset file permissions? [y/N] ").lower() == 'y':
                    try:
                        # Reset permissions
                        os.chmod(pem_file, 0400)
                    except:
                        sys.stderr.write("Unable to reset file permissions!\n")
                        if raw_input("Do you wish to continue? [y/N] ").lower() != 'y':
                            sys.exit(1)
                else:
                    print "File permissions unchanged."
                    if raw_input("Do you wish to continue? [y/N] ").lower() != 'y':
                        sys.exit(1)
        else:
            sys.stderr.write("WARNING: The created pem file no longer exists at %s!\n" % pem_file)
            sys.stderr.write("         Please copy it out of the previous 'pem_home' directory.\n\n")

            if noprompts or raw_input("Do you wish to continue? [y/N] ").lower() != 'y':
                sys.exit(1)

    except boto.exception.EC2ResponseError, e:
        if e.code == 'InvalidKeyPair.NotFound':
            print 'Creating keypair...'
            key = conn.create_key_pair(key_pair)
            try:
                key.save(pem_home)
            except:
                conn.delete_key_pair(key_pair)
                print "Saving keypair failed! Removing keypair from AWS and exiting. Nothing was launched.\n"
                raise
        else:
            raise

    # Check if DataStax security group exists
    ds_security_group = False
    for security_group in conn.get_all_security_groups():
        if security_group.name == 'DataStax':
            ds_security_group = security_group
            break

    # Create the DataStax security group if it doesn't exist
    if not ds_security_group:
        ds_security_group = conn.create_security_group('DataStax', 'Security group for running DataStax products')
    
    # Ensure all Security settings are active
    print "Configuring ports..."
    authorize(ds_security_group, 22, 'public')
    authorize(ds_security_group, 9160, 'public')
    authorize(ds_security_group, 8012, 'public')
    authorize(ds_security_group, 50030, 'public')
    authorize(ds_security_group, 50060, 'public')
    authorize(ds_security_group, 8983, 'public')
    authorize(ds_security_group, 8888, 'public')

    authorize(ds_security_group, 7000, 'private')
    authorize(ds_security_group, 7199, 'private')
    authorize(ds_security_group, 8983, 'private')
    authorize(ds_security_group, 9290, 'private')
    authorize(ds_security_group, 61621, 'private')
    authorize(ds_security_group, 1024, 'private', 65535)
    authorize(ds_security_group, 50031, 'private')
    authorize(ds_security_group, 61620, 'private')

    try:
        # Create the EC2 cluster
        print 'Launching cluster...'
        try:
            reservation = conn.run_instances(image, 
                                             min_count=reservation_size, 
                                             max_count=reservation_size, 
                                             instance_type=instance_type, 
                                             key_name=key_pair, 
                                             placement=placement, 
                                             security_groups=['DataStax'], user_data=user_data)
        except boto.exception.EC2ResponseError:
            print_boto_error()

        # Sleep so Amazon recognizes the new instance
        print 'Waiting for EC2 cluster to instantiate...'
        time.sleep(10)
        for instance in reservation.instances:
            while not instance.update() == 'running':
                time.sleep(3)
        print "Cluster booted successfully!"
        print

        # Tag the instances in this reservation
        for instance in reservation.instances:
            conn.create_tags([instance.id], {'Name': tag, 'Initializer': 'DataStax'})
    except:
        print "\n\nERROR: Tags were never applied to started instances!!! Make sure you TERMINATE instances here:"
        print "    https://console.aws.amazon.com/ec2/home?region=us-east-1#s=Instances\n"
        
        # Ensure the user acknowledges pricing danger
        while raw_input('Acknowledge warning [Type OK]: ').lower() != 'ok':
            pass
        raise

    # Collect ip addresses
    private_ips = []
    public_ips = []
    for instance in reservation.instances:
        private_ips.append(instance.private_ip_address)
        public_ips.append(instance.ip_address)

    return [private_ips, public_ips, reservation]

def terminate_cluster(aws_access_key_id, aws_secret_access_key, search_term, prompt_continuation=False):

    # Grab all the infomation for clusters spawn by this tool that are still alive
    ds_reservations = {}
    conn = boto.connect_ec2(aws_access_key_id, aws_secret_access_key)

    try:
        reservations = conn.get_all_instances()
    except boto.exception.EC2ResponseError:
        print_boto_error()

    for reservation in reservations:
        if 'Initializer' in reservation.instances[0].tags and reservation.instances[0].tags['Initializer'] == 'DataStax' and reservation.instances[0].update() == 'running':
            if not reservation.instances[0].tags['Name'] in ds_reservations and search_term.lower() in reservation.instances[0].tags['Name'].lower():
                ds_reservations[reservation.instances[0].tags['Name']] = {
                    'Reservation': reservation
                }

    if not ds_reservations.keys():
        print "No existing clusters currently running!"
        print
        return

    # Prompt for cluster to destroy
    selection = common.choose("Choose the cluster to destroy (if you wish):", ds_reservations.keys(), noneOption=True)

    # Return if you do not with to kill a cluster
    if selection == 'None':
        return

    # Confirm cluster termination
    print "Confirm you wish to terminate {0} by pressing 'y'.".format(selection)
    if raw_input().lower() != 'y':
        print "Cluster was not terminated."
        print
        sys.exit()
    print

    # The actual termination command
    for instance in ds_reservations[selection]['Reservation'].instances:
        conn.terminate_instances([instance.id])

    print "Termination command complete."
    print

    if prompt_continuation:
        if raw_input('Do you wish to launch another cluster? [Y/n] ').lower() == 'n':
            sys.exit(0)
        print



#!/usr/bin/env python

class CassandraLauncher:
    def __init__(self):
        import cassandralauncher
        cassandralauncher.run()

class ClusterLauncher:
    def __init__(self):
        import clusterlauncher
        clusterlauncher.run()



#!/usr/bin/env python

import sys

if sys.version_info < (2, 6):
      sys.stderr.write('Please install (and use) Python2.6, or greater, to run setup.py.\n')
      sys.exit(1)

try:
      from setuptools import setup
except:
      from distutils.core import setup

long_description = """CassandraLauncher contains two parts:

1. `cassandralauncher` is accessible straight from the command line and launches a Cassandra cluster for you.
2. `clusterlauncher` is also accessible from the command line and provides a user interface for AWS and RAX.

Both run off of a config: clusterlauncher.conf.
"""

setup(name='CassandraLauncher',
      version='1.7-0',
      description='Command line utilities for launching Cassandra clusters in EC2',
      long_description=long_description,
      author='Joaquin Casares',
      author_email='joaquin.casares AT gmail.com',
      url='http://www.github.com/joaquincasares/cassandralauncher',
      packages=['cassandralauncher'],
      scripts=['scripts/cassandralauncher', 'scripts/clusterlauncher'],
      package_data={'': ['README.md']},
      data_files=[('/etc/cassandralauncher', ['cassandralauncher/clusterlauncher.conf'])],
      keywords="apache cassandra datastax cluster clustertools cloud cloudservers rackspace ec2 aws on-demand",
      install_requires=["boto", "python-cloudservers"]
     )
